
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "UART.h"

//-----------------------------------------------------------------------------------------

void uart_init(void)
{
  UBRRH = UBRRH_VALUE;                              // wird von setbaud.h gesetzt 
  UBRRL = UBRRL_VALUE;                              // wird von setbaud.h gesetzt 
#if USE_2X                                          // wird von setbaud.h gesetzt
  UCSRA =	(1<<U2X);
#endif
  UCSRB =	(1<<TXEN) | (1<<RXEN);
  UCSRC =	(1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);
}

//-----------------------------------------------------------------------------------------

void uart_send_byte(uint8_t zeichen)
{
  while (!(UCSRA & (1<<UDRE)))                      // warten bis UDRE 1 
    {
    }
  UDR = zeichen;
}

//-----------------------------------------------------------------------------------------

void uart_send_text_p(const char *text)
{
  uint8_t zeichen;

  while ((zeichen = pgm_read_byte(text)))
    { 
      uart_send_byte(zeichen);
      text++;
    }
}

//-----------------------------------------------------------------------------------------

void uart_send_text(const char *text)
{
  uint8_t zeichen;

  while ((zeichen = *text))
   { 
     uart_send_byte(zeichen);
     text++;
   }
}

//-----------------------------------------------------------------------------------------

void uart_send_byte_hex(uint8_t byte)
{
  uint8_t zeichen;

  zeichen = '0' | ((byte & 0xF0) >> 4);
  if (zeichen > '9')
    {
      zeichen += 7;
    }
  uart_send_byte(zeichen);

  zeichen = '0' | (byte & 0x0F);
  if (zeichen > '9')
    {
      zeichen += 7;
    }
  uart_send_byte(zeichen);
}

//-----------------------------------------------------------------------------------------

void uart_send_byte_bin(uint8_t byte)
{
  uint8_t zeichen, i;

  for (i=0; i<8; i++)
    {
      if (byte & 0x80)
        {
          zeichen = '1';
        }
      else
        {
          zeichen = '0';
        }
      byte = (byte<<1);
      if (i == 4)
        {
          uart_send_byte(' ');
        }  
      uart_send_byte(zeichen);
    }
}

//-----------------------------------------------------------------------------------------
